package de.pfeifer_syscon.corvid;

import COSE.OneKey;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Base64;
import java.util.Date;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author rpf
 */
public class TrustedKey implements Serializable {

    String certificateType;
    String country;
    String kid;
    String rawData;
    String signature;
    String thumbprint;
    String timestamp;

    public TrustedKey(String skey) throws Exception {
        JSONObject jkey = (JSONObject) (new JSONParser().parse(skey));
        this.certificateType = (String) jkey.get("certificateType");
        this.country = (String) jkey.get("country");
        this.kid = (String) jkey.get("kid");
        this.rawData = (String) jkey.get("rawData");
        this.signature = (String) jkey.get("signature");
        this.thumbprint = (String) jkey.get("thumbprint");
        this.timestamp = (String) jkey.get("timestamp");
    }

    public boolean matching(String country, byte[] kid) {
        byte[] thisKid = Base64.getDecoder().decode(this.kid);
        return this.country.equals(country) // CovPass does not check country but allows multiple kid matches ...
            && Arrays.equals(thisKid, kid);
    }

    public OneKey getKey() throws Exception {
        byte[] keyBytes = Base64.getDecoder().decode(this.rawData);
        CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
        InputStream in = new ByteArrayInputStream(keyBytes);
        X509Certificate cert = (X509Certificate) certFactory.generateCertificate(in);
        Date now = new Date();
        // alternative cert.checkValidity(); which handles utc ... 
        if (!(now.after(cert.getNotBefore()) &&
            now.before(cert.getNotAfter()))) {
            throw new Exception(String.format("The certificat is valid from %tF to %tF which does not include %tF",
                cert.getNotBefore(), cert.getNotAfter(), now));
        }        
        // Requires latest COSE version
        OneKey oneKey = new OneKey(cert.getPublicKey(), null);
        return oneKey;
    }
}
