/*
 */
package de.pfeifer_syscon.qr;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.Result;
import com.google.zxing.client.j2se.BufferedImageLuminanceSource;
import com.google.zxing.client.j2se.MatrixToImageWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.QRCodeWriter;
import de.pfeifer_syscon.corvid.CertInterface;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Map;
import javax.imageio.ImageIO;

public class QRHandler {

    /**
     * convert information to qr, the charset is handled by qr as far as i can see
     * @param certMessage
     * @param type image output type e.g. "png"
     * @param qrCodePixel number of output pixels (surprisingly on reread fewer work better ....)
     * @param writeHintMap the hints e.g. error_correction ...
     * @return a byte error with encoded image
     * @throws Exception if something goes wrong
     */
    public byte[] write(CertInterface certMessage, String type, int qrCodePixel, Map<EncodeHintType, Object> writeHintMap) throws Exception {
        
        BitMatrix matrix = new QRCodeWriter().encode(
                certMessage.getRawString(), 
                BarcodeFormat.QR_CODE, qrCodePixel, qrCodePixel, writeHintMap);
        //BitMatrix matrix = new MultiFormatWriter().encode(
        //        new String(certMessage.getRawString().getBytes(charset), charset),  // see if all chars match charset
        //        BarcodeFormat.QR_CODE, qrCodePixel, qrCodePixel, writeHintMap);

        try (ByteArrayOutputStream os = new ByteArrayOutputStream(8192)) {
            MatrixToImageWriter.writeToStream(matrix, type, os);   // png shoud give best qualtity on b/w
            os.flush();
            return os.toByteArray();
        }
    }

    /**
     * @param filePath
     * @param charset
     * @return Qr Code value
     * @throws Exception
     */
    public String read(File filePath) throws Exception {
        try (InputStream fis = new FileInputStream(filePath)) {
            BufferedImageLuminanceSource bils = new BufferedImageLuminanceSource(ImageIO.read(fis));
            HybridBinarizer hib = new HybridBinarizer(bils);
            BinaryBitmap binaryBitmap = new BinaryBitmap(hib);
            Result qrCodeResult = new MultiFormatReader().decode(binaryBitmap);
            //Result qrCodeResult = new QRCodeReader().decode(binaryBitmap);

            String txt = qrCodeResult.getText();
            return txt;
        }
    }

    public static String debug(byte[] min) {
        StringBuilder sb = new StringBuilder();
        for (int a = 0; a < 0x200 && a < min.length; a += 0x10) {
            sb.append(String.format("%04x ", a));
            for (int o = 0; o < 0x10; ++o) {
                if (a+o < min.length) {
                    sb.append(String.format("%02x ", min[a+o] & 0xff));
                }
                else {
                    sb.append("   ");
                }
            }
            sb.append("   ");
            for (int o = 0; o < 0x10; ++o) {
                if (a+o < min.length) {
                int val = min[a+o] & 0xff;
                    sb.append(String.format("%c", (val >= 32 && val < 254) ? (char)val : '.'));
                }
                else {
                    sb.append(" ");
                }
            }
            sb.append('\n');
        }
        return sb.toString();
    }

}
